--[[ Slash Commands ]]

local _G = _G
local expWod = select(4, GetBuildInfo()) >= 60000
local expMop = select(4, GetBuildInfo()) >= 50000 and select(4, GetBuildInfo()) < 60000
local expCat = select(4, GetBuildInfo()) >= 40000 and select(4, GetBuildInfo()) < 50000
local fixRModules = {};
local fixRModules_LoadedCallbacks = {};
FixR = {};

local loadModule = nil
local unloadModule = nil
local fixR_isModuleLoaded = nil
local fixR_callModuleLoadedCallbacks = nil
local FIXR_VERSION = GetAddOnMetadata("FixR", "Version");
local DefaultFixROptions = {
  ["FixRVersion"] = FIXR_VERSION;
  ["Modules"] = {};
};
function FixR_FreshOptions()
  FixROptions = FixR_CloneTable(DefaultFixROptions);
end
function FixR_UpdateOptionsArray()
  local TempFixROptions = FixR_CloneTable(DefaultFixROptions);
  FixR_CopyValues(FixROptions,TempFixROptions);
  FixROptions = TempFixROptions
end
function FixR_ShowValues(t,appx) -- Print array
  if t then
    local i, v = next(t, nil); -- i is an index of t, v = t[i]
    while i do
      if type(v)=="table" then
        print(appx.."'"..i.."' = {");
        FixR_ShowValues(v,"  "..appx);
        print(appx.."};");
      elseif type(v)=="boolean" then
        print(appx.."'"..i.."' = "..(v and "true" or "false")..";");
      elseif type(v)=="function" then
        print(appx.."'"..i.."' = <function>;");
      elseif v==nil then
        print(appx.."'"..i.."' = <nil>;");
      else
        print(appx.."'"..i.."' = "..v..";");
      end
      i, v = next(t, i); -- get next index
    end
  else
    print(appx.."nil");
  end
end
function FixR_CopyValues(from,to)
  local i, v = next(from, nil); -- i is an index of from, v = from[i]
  while i do
    if type(v)=="table" then
      if to[i] ~= nil then
        if type(to[i])=="table" then
          FixR_CopyValues(v,to[i]);
        end
      end
    else
      if to[i] ~= nil then
        if type(to[i])~="table" then
          to[i] = v;
        end
      end
    end
    i, v = next(from, i); -- get next index
  end
end
--from "SCT" by "Grayhoof"
function FixR_CloneTable(t) -- return a copy of the table t
  local new = {}; -- create a new table
  local i, v = next(t, nil); -- i is an index of t, v = t[i]
  while i do
    if type(v)=="table" then
      v=FixR_CloneTable(v);
    end
    new[i] = v;
    i, v = next(t, i); -- get next index
  end
  return new;
end
local function mysplit(inputstr, sep)
  if sep == nil then
    sep = "%s"
  end
  local t = {}
  local i = 1
  for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
    t[i] = str
    i = i + 1
  end
  return t
end
local function round(num, idp)
  if idp and idp>0 then
    local mult = 10^idp
    return math.floor(num * mult + 0.5) / mult
  end
  return math.floor(num + 0.5)
end
local function fixR_NewButton(parent, w, h)
  local f = CreateFrame("Button", nil, parent)
  --QRSFrame:SetPoint("CENTER", parent, "CENTER", 0, 0)
  f:SetFrameStrata("DIALOG")
  f:SetWidth(w)
  f:SetHeight(h)
  
  --QRSFrame:SetText("Choose for me")
  f:SetNormalFontObject("GameFontNormal")
  
  local ntex = f:CreateTexture()
  ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
  ntex:SetTexCoord(0, 0.625, 0, 0.6875)
  ntex:SetAllPoints()	
  f:SetNormalTexture(ntex)
  
  local htex = f:CreateTexture()
  htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
  htex:SetTexCoord(0, 0.625, 0, 0.6875)
  htex:SetAllPoints()
  f:SetHighlightTexture(htex)
  
  local ptex = f:CreateTexture()
  ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
  ptex:SetTexCoord(0, 0.625, 0, 0.6875)
  ptex:SetAllPoints()
  f:SetPushedTexture(ptex)
  
  return f
end

function FixR:NewModule(name, defaultO, misc)
  if not fixRModules[name] then
    fixRModules[name] = {};
    DefaultFixROptions["Modules"][name] = FixR_CloneTable(defaultO or {})
    DefaultFixROptions["Modules"][name]["enabled"] = false
    fixRModules[name].misc = misc
  end
  return fixRModules[name]
end

function FixR:Echo(...)
  fixR_Echo(...)
end

function FixR:IsCat()
  return expCat
end
function FixR:IsMop()
  return expMop
end
function FixR:IsWod()
  return expWod
end
function FixR:Round(num, idp)
  return round(num, idp)
end
function FixR:SetModuleGUIOption(m, opt, val)
  if m.misc and m.misc["GUI"] and m.misc["GUI"][opt] then
    local optType = m.misc["GUI"][opt]["type"]
    if optType == "Boolean" then
      m.GUIoption[opt]:SetChecked(val)
    end
  end
end

local function fixR_toggleModule(module, blnLoad)
  if blnLoad then --not yet loaded
    loadModule(module)
    fixR_Echo(module.." |cffaaffaaenabled")
  else --loaded
    unloadModule(module)
    fixR_Echo(module.." |cffff8888disabled")
  end
end

local fixRFrame = CreateFrame("Frame")
local fixROptionsFrame = CreateFrame("Frame", "fixROptionsFrame", UIParent)

local fixRFrameEvents = {};
function fixRFrameEvents:PLAYER_ENTERING_WORLD(...)
  fixRFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
  if ( FixROptions == nil) then
    fixR_Echo("FixR: Fresh install.")
    FixR_FreshOptions();
  elseif FixROptions["FixRVersion"] ~= FIXR_VERSION then
    fixR_Echo("FixR: New version installed. Check your options! (old: ".. FixROptions["FixRVersion"] .. ", new: " .. FIXR_VERSION .. ")")
    FixR_UpdateOptionsArray();
    FixROptions["FixRVersion"] = FIXR_VERSION
  else
    FixR_UpdateOptionsArray();
  end
  
  --setup config dialog
  fixROptionsFrame.name = "FixR"
  InterfaceOptions_AddCategory(fixROptionsFrame)
  
  local fontStringVersion = fixROptionsFrame:CreateFontString(nil, "ARTWORK", "GameTooltipText")
  fontStringVersion:SetPoint("TOPRIGHT", -10, -20)
  fontStringVersion:SetText("FixR v|cffaaffaa"..FIXR_VERSION)
  
  local fontString1 = fixROptionsFrame:CreateFontString(nil, "ARTWORK")
  fontString1:SetFont("Fonts\\FRIZQT__.TTF", 14)
  fontString1:SetPoint("TOPLEFT", 20, -10)
  fontString1:SetText("FixR Modules:")
  fontString1:SetHeight(30)
  local fontString2 = fixROptionsFrame:CreateFontString(nil, "ARTWORK")
  fontString2:SetFont("Fonts\\FRIZQT__.TTF", 11)
  fontString2:SetPoint("TOPLEFT", fontString1, "BOTTOMLEFT", 0, -3)
  fontString2:SetText("|cffaaaaaaGrayed out modules are in alpha stage.|r")
  fontString2:SetHeight(35)
  
  local lastCheckButton = fontString2
  
  --load modules
  for k, v in pairs(fixRModules) do
    --create checkButton for GUI
    v.GUIcheck = CreateFrame("CheckButton", "FixR_GUI_checkButton"..k, fixROptionsFrame, "ChatConfigCheckButtonTemplate")
    if lastCheckButton then
      v.GUIcheck:SetPoint("TOPLEFT", lastCheckButton, "BOTTOMLEFT", 0, -3)
    else
      v.GUIcheck:SetPoint("TOPLEFT", 20, -20)
    end
    lastCheckButton = v.GUIcheck
    _G[(v.GUIcheck:GetName()).."Text"]:SetText(k)
    
    local misc = v.misc
    if misc then
      v.GUIcheck.tooltip = misc["desc"]
      if misc["alpha"] then
        v.GUIcheck:SetAlpha(0.5)
      end
    else
      v.GUIcheck.tooltip = "Tooltip-Text fuer "..k
    end
    
    v.GUIcheck.tag = k
    v.GUIcheck:SetChecked(FixROptions["Modules"][k]["enabled"])
    v.GUIcheck:SetScript("OnClick", function(self)
      fixR_toggleModule(self.tag, self:GetChecked())
    end)
    
    --load module
    if FixROptions["Modules"][k] and FixROptions["Modules"][k]["enabled"] then
      loadModule(k)
    end
  end
end
function fixRFrameEvents:PLAYER_LOGOUT(...)
  --update module options
  for k, v in pairs(fixRModules) do
    if v["Logout"] then
      local ok, err = pcall(v["Logout"], v)
      if not ok then
        fixR_Echo("|cffff8888Error while calling func Logout in module ", k, "|cffff8888: "..err)
      end
    end
    if (type(FixROptions["Modules"][k]) == "table") and (type(v.O) == "table") and (v.LoadSuccess) then --v.O is not a table (nil) if the module had not been loaded
      FixROptions["Modules"][k] = FixR_CloneTable(v.O)
    end
  end
end
fixRFrame:SetScript("OnEvent", function(self, event, ...)
  fixRFrameEvents[event](self, ...); -- call one of the functions above
end);
for k, v in pairs(fixRFrameEvents) do
  fixRFrame:RegisterEvent(k); -- Register all events for which handlers have been defined
end

function loadModule(name)
  if fixRModules[name] then
    local m = fixRModules[name]
    --update modul options
    m.O = FixR_CloneTable(FixROptions["Modules"][name])
    m.O["enabled"] = true
    FixROptions["Modules"][name]["enabled"] = true

    local ok, err = true, nil
    m.LoadSuccess = true
    if m["Load"] then
      ok, err = pcall(m["Load"], m)
      if not ok then
        fixR_Echo("|cffff8888Error while loading module ", name, "|cffff8888: "..err)
        m.LoadSuccess = false
      end
    end
    
    m.GUIcheck:SetChecked(ok)
    
    if ok and (not m.GUIOptionsFrame) then
      --add sub-category
      local f = CreateFrame("Frame", nil, fixROptionsFrame)
      m.GUIOptionsFrame = f
      f.name = name
      f.parent = fixROptionsFrame.name
      InterfaceOptions_AddCategory(f)
      
      local fs1 = f:CreateFontString(nil, "ARTWORK")
      fs1:SetFont("Fonts\\FRIZQT__.TTF", 14)
      fs1:SetPoint("TOPLEFT", 20, -10)
      if m.misc["alpha"] then
        fs1:SetText("FixR / |cffaaaaaa"..name.."|r")
      else
        fs1:SetText("FixR / "..name)
      end
      fs1:SetHeight(30)
      
      local last = fs1
      
      --add options to sub-category
      if m.misc and m.misc["GUI"] then
        m.GUIoption = {}
        for i, v in ipairs(m.misc["GUI"]) do
          if v["type"] == "Boolean" then
            m.GUIoption[i] = CreateFrame("CheckButton", "FixR_moduleGUI_"..name.."_"..i, f, "ChatConfigCheckButtonTemplate")
            m.GUIoption[i]:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -3)
            last = m.GUIoption[i]
            _G[(last:GetName()).."Text"]:SetText(v["opt"])
            last.tooltip = v["desc"]
            
            last:SetChecked(m.O[v["opt"]])
            
            last.tagModuleName = name
            last.tagOption = v["opt"]
            last:SetScript("OnClick", function(self)
              if fixRModules[self.tagModuleName].O["enabled"] then
                if fixRModules[self.tagModuleName]["SetOption"] then
                  fixRModules[self.tagModuleName]:SetOption(self.tagOption, self:GetChecked() and 1 or 0)
                end
              else
                self:SetChecked(fixRModules[self.tagModuleName].O[self.tagOption])
              end
            end)
          elseif v["type"] == "Integer" then
          --FixR_moduleGUI_testmodul_2:SetBackdropBorderColor(0,0,0,0.4)
          --/run _G["FixR_moduleGUI_testmodul_3"]:SetTextInsets(0,0,3,3)
            m.GUIoption[i] = CreateFrame("EditBox", "FixR_moduleGUI_"..name.."_"..i, f, "InputBoxTemplate")
            m.GUIoption[i]:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -3)
            last = m.GUIoption[i]
            last:SetAutoFocus(false)
            last:SetWidth(122)
            last:SetHeight(22)
            last:SetTextInsets(0,0,3,3)
            last:SetNumeric()
            last.tooltip = v["desc"]
            last.optMin = v["min"]
            last.optMax = v["max"]
            last.optValue = m.O[v["opt"]]
            --last.optDefaultValue = m.DefaultO[v["opt"]]
            
            last:SetNumber(m.O[v["opt"]])
            last:SetCursorPosition(0)
            
            last.tagModuleName = name
            last.tagOption = v["opt"]
            last:SetScript("OnEnterPressed", function(self)
              self:ClearFocus()
              local n = self:GetText() == "" and self.optValue or self:GetText()
              n = max(n, self.optMin)
              n = min(n, self.optMax)
              self:SetNumber(n)
              if fixRModules[self.tagModuleName].O["enabled"] then
                if fixRModules[self.tagModuleName]["SetOption"] then
                  self.optValue = self:GetText() or self.optMin
                  fixRModules[self.tagModuleName]:SetOption(self.tagOption, self.optValue)
                end
              else
                self:SetNumber(self.optValue)
              end
            end);
            last:SetScript("OnEscapePressed", function(self)
              self:ClearFocus()
              self:SetNumber(self.optValue)
            end);
            last:SetScript("OnTextChanged", function(self)
            end);
          elseif v["type"] == "Button" then
            m.GUIoption[i] = fixR_NewButton(f, 120, 22)
            m.GUIoption[i]:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -3)
            last = m.GUIoption[i]
            last:SetText(v["opt"])
            
            last.tagModuleName = name
            last.tagOption = v["opt"]
            last.func = v["func"]
            last:SetScript("OnClick", function(self)
              if self.func then
                self.func(fixRModules[self.tagModuleName])
              end
            end)
          end
        end
      end
    end
    
    if m.LoadSuccess then
      fixR_callModuleLoadedCallbacks(name)
    end
  end
end
function unloadModule(name)
  if fixRModules[name] then
    local m = fixRModules[name]
    
    m.O["enabled"] = false
    FixROptions["Modules"][name]["enabled"] = false
    
    if m["Unload"] then
      local ok, err = pcall(m["Unload"])
      if not ok then
        fixR_Echo("|cffff8888Error while unloading module ", name, "|cffff8888: "..err)
      end
    end
    
    m.GUIcheck:SetChecked(false)
    
    if m.GUIOptionsFrame then
      --cannot remove category
    end
  end
end
function fixR_isModuleLoaded(name)
  return FixROptions["Modules"][name] and FixROptions["Modules"][name]["enabled"] or false
end

function FixR:IsModuleLoaded(name)
  return fixR_isModuleLoaded(name)
end
function FixR:RegisterModuleLoadedCallback(name, m)
  if not fixRModules_LoadedCallbacks[name] then
    fixRModules_LoadedCallbacks[name] = {}
  end
  tinsert(fixRModules_LoadedCallbacks[name], m)
end
function fixR_callModuleLoadedCallbacks(name)
  if fixRModules_LoadedCallbacks[name] and type(fixRModules_LoadedCallbacks[name]) == "table" then
    local t = fixRModules_LoadedCallbacks[name]
    local i, v = next(t, nil); -- i is an index of t, v = t[i]
    while i do
      if v and v["ModuleLoadedCallback"] then
        local ok, err = pcall(v["ModuleLoadedCallback"], v, name)
        if not ok then
          fixR_Echo("|cffff8888Error while calling func ModuleLoadedCallback in module ", name, "|cffff8888: "..err)
        end
      end
      i, v = next(t, i); -- get next index
    end
    fixRModules_LoadedCallbacks[name] = {}
  end
end

function fixR_Echo(...)
  local t = {...}
  if #t > 0 then
    DEFAULT_CHAT_FRAME:AddMessage("|cffaaaaff"..table.concat(t, "|cffaaaaff").."|r")
  else
    DEFAULT_CHAT_FRAME:AddMessage("|cffff0000nil|r")
  end
end

SLASH_FIXR1 = "/fixr" --SLASH_FIXR2 = "/fixing"
SlashCmdList["FIXR"] = function(msg, editbox)
  msg = msg or ""
  args = mysplit(msg, " ")
  local isHandled = false
  for k, v in pairs(fixRModules) do
    if not isHandled and string.lower(k) == string.lower(args[1] or "") then
      isHandled = true
      if args[2] ~= nil then -- only SetOption if option is given
        if fixR_isModuleLoaded(k) then -- only SetOption if module is loaded
          if args[3] ~= nil then
            if v and v["SetOption"] then
              v:SetOption(args[2], args[3])
            end
          else
            if v and v["SetOption"] then
              v:SetOption(args[2])
            end
          end
        end
      else
        local loaded = FixROptions["Modules"][k]["enabled"]
        loaded = not (loaded or false)
        fixR_toggleModule(k, loaded)
      end
    end
  end
  if not isHandled then
    --[[
    fixR_Echo("FixR v|cffaaffaa"..FIXR_VERSION)
    for k, v in pairs(FixROptions["Modules"]) do
      local bln = v["enabled"]
      fixR_Echo(k..(bln and ": |cffaaffaaEnabled" or ": |cffff8888Disabled"))
      if bln then
        if fixRModules[k] and fixRModules[k]["ShowOptions"] then
          fixRModules[k]:ShowOptions()
        end
      end
    end
    --]]
    InterfaceOptionsFrame_OpenToCategory(fixROptionsFrame)
    InterfaceOptionsFrame_OpenToCategory(fixROptionsFrame) --yes, two times
  end
end

--/script print(GetMouseFocus():GetName())

FixRMount = function()
  --pre 6.0: if GetZoneText()=="Timeless Isle"then local M="MOUNT";for i=1,GetNumCompanions(M)do if select(3,GetCompanionInfo(M,i))==148428 then CallCompanion(M,i)end end end
  --if GetZoneText()=="Timeless Isle"then local j,s=C_MountJournal;for i=1,j.GetNumMounts()do _,s=j.GetMountInfo(i)if s==148428 then j.Summon(i)end end end
  if GetZoneText()=="Timeless Isle"then
    local j,s=C_MountJournal
    for i=1,j.GetNumMounts() do
      --local creatureName, spellID, icon, active, isUsable, sourceType, isFavorite, _, _, hideOnChar, isCollected = j.GetMountInfo(i)
      _,s=j.GetMountInfo(i)
      if s==148428 then
        j.Summon(i)
        return
      end
    end
  end
end